/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse, die ein Farbattribut ein- und auslesen kann
*/
function AttributeColor(
    attributeName,      // Attributname
    allowNone,          // Optional: Auch keine Farbe erlauben?
    required            // Optional: Muss dieses Attribut angegeben sein?
) {
    Object.defineProperty(this, 'attributeName', { 'value': attributeName, 'enumerable': true });
    if (allowNone)
        this.allowNone = true;
    if (required)
        this.required = true;
}

// Attributname
AttributeColor.prototype.attributeName = 'farbe';

// Muss dieses Attribut angegeben sein?
AttributeColor.prototype.required = false;

// Auch keine Farbe erlauben?
AttributeColor.prototype.allowNone = false;

// Standardwert
AttributeColor.prototype.defaultValue = 'none';

// Alle normalen Farbwerte
AttributeColor.VALUES = [
    { value: '#c00', key: 'rot' },
    { value: '#f57900', key: 'orange' },
    { value: '#edd400', key: 'gelb' },
    { value: '#73d216', key: 'grün' },
    { value: '#3465a4', key: 'blau' },
    { value: '#75507b', key: 'violett' },
    { value: '#c17d11', key: 'braun' },
    { value: '#eeeeec', key: 'weiß' },
    { value: '#888a85', key: 'grau' },
    { value: '#2e3463', key: 'schwarz' }
];

// Erlaubte Angaben für „keine Farbe“
AttributeColor.KEYSNONE = ['keine', 'transparent'];

// Bezeichnung für unbekannte Farbangabe
AttributeColor.KEYUNKNOWN = 'unbekannt';

/**
* Zum zugehörigen Attributwert in obj die Bezeichnung ermitteln
* @return String        Bezeichnung des Werts (false, wenn kein Attributwert angegeben)
*/
AttributeColor.prototype.nameValue = function(
    obj     // Objekt mit den Attributwerten
) {
    var value = obj[this.attributeName];
    if (value === undefined)
        return false;
    for (var n = 0; n < AttributeColor.VALUES.length; ++n) {
        if (value == AttributeColor.VALUES[n].value)
            return AttributeColor.VALUES[n].key;
    }
    if (AttributeColor.KEYSNONE.indexOf(value) != -1)
        return AttributeColor.KEYSNONE[0];
    return AttributeColor.KEYUNKNOWN;
}

/**
* Den zur Angabe strKey passenden Attributwert im Objekt obj eintragen
* @return String        Im Fehlerfall Fehlermeldung
*/
AttributeColor.prototype.readValue = function(
    strKey,     // Von Benutzer eingegebener Wert, der tatsächlichen Attributwert beschreibt
    obj         // Objekt für tatsächliche Attributwerte
) {
    if (!strKey && this.required)
        return 'Wert ' + (this.attributeName ? ('des Attributs „' + this.attributeName + '“ ') : '') + 'fehlt';
    if (!strKey) {
        obj[this.attributeName] = this.defaultValue;
        return;
    }
    var cleanedKey = strKey.toLowerCase();
    var index = AttributeColor.VALUES.map(function(item) { return item.key; }).indexOf(cleanedKey);
    if (index != -1) {
        obj[this.attributeName] = AttributeColor.VALUES[index].value;
        return;
    }
    if (this.allowNone && (AttributeColor.KEYSNONE.indexOf(cleanedKey) != -1)) {
        obj[this.attributeName] = 'none';
        return;
    }
    var arrValid = AttributeColor.VALUES.map(function(item) { return item.key; });
    if (this.allowNone)
        arrValid.unshift(AttributeColor.KEYSNONE[0]);
    return 'Ungültiger Wert „' + strKey + (this.attributeName ? ('“ für Attribut „' + this.attributeName) : '') + '“ (Erlaubte Werte: ' + arrValid.join(', ') + ')';
}
