/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Daten einer anzeigbaren Linie
*/
function LineDataBuilder() {
    this._attributes = [
        new AttributeColor('farbe', true),
        new AttributeNumber('strichdicke', 0, 10, 1),
        new AttributeNumber('startX', -1, 10, 0),
        new AttributeNumber('startY', -1, 6, 0),
        new AttributeNumber('endX', -1, 10, 0),
        new AttributeNumber('endY', -1, 6, 0),
        new AttributeSet('linienenden', [ { key: 'abgeschnitten', value: 'butt' }, { key: 'rund', value: 'round' }, { key: 'quadratisch', value: 'square' } ]),
        new AttributeSet('linienstil', [ { key: 'durchgezogen', value: 'none' }, { key: 'gepunktet', value: '1 1' }, { key: 'gestrichelt', value: '3 3' } ])
    ];
    this._attributes[1].defaultValue = 0.1;
}

// Alle Attribute
LineDataBuilder.prototype._attributes = [];

/**
* Alle Attributnamen ermitteln
* @return Array     Attributnamen als Strings
*/
LineDataBuilder.prototype.getAttributeNames = function() {
    return this._attributes.map(function(item) { return item.attributeName; });
}

/**
* Eingegebene Attributwerte in tatsächliche Attributwerte umwandeln
* @return Object    Objekt mit Attributen message (Fehlermeldung, falls vorhanden) und object (Objekt mit tatsächlichen Attributwerten)
*/
LineDataBuilder.prototype.createFromAtts = function(
    atts        // Eingegebene Attributwerte
) {
    var obj = {};
    var s;
    for (var n = 0; n < this._attributes.length; ++n) {
        if (s = this._attributes[n].readValue(atts[this._attributes[n].attributeName], obj))
            return { message: s, object: obj };
    }
    return { message: '', object: obj };
}
