/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse, um aus einem Objekt mit tatsächlichen Attributwerten einen entsprechenden DOM-Knoten zu machen
*/
function LineNodeBuilder() {
}

/**
* Erstellt einen DOM-Knoten anhand von Attributwerten
* @return SVGElement    Erstellter DOM-Knoten
*/
LineNodeBuilder.prototype.create = function(
    atts        // Objekt mit den tatsächlichen Attributwerten
) {
    var offset = (atts.linienende && (atts.linienende != 'butt')) ? 0.9 : 0;
    var dasharray = atts.linienstil ? atts.linienstil.split(' ').map(function(v, index) { return Number.isFinite(+v) ? ((+v + ((index % 2) ? offset : (-offset))) * atts.strichdicke * 10) : v; }).join(' ') : 'none';
    return ScoTools.createSvg('line', undefined, {
        x1: (atts.startX + 1) * 10,
        y1: (atts.startY + 1) * 10,
        x2: (atts.endX + 1) * 10,
        y2: (atts.endY + 1) * 10,
        'stroke-linecap': atts.linienenden,
        fill: 'none',
        stroke: atts.farbe,
        'stroke-width': atts.strichdicke * 10,
        'stroke-dasharray': dasharray
    });
}

/**
* Erstellt einen DOM-Knoten anhand von Attributwerten und ersetzt nodeOld durch den neu erstellten Knoten bzw. fügt ihn am Ende von parentNode ein
* @return SVGElement    Erstellter DOM-Knoten
*/
LineNodeBuilder.prototype.change = function(
    atts,       // Objekt mit den tatsächlichen Attributwerten
    nodeOld,    // Zu ersetzender DOM-Knoten
    parentNode  // Übergeordneter DOM-Knoten
) {
    var node = this.create(atts);
    if (nodeOld && nodeOld.parentNode)
        nodeOld.parentNode.replaceChild(node, nodeOld);
    else
        parentNode.appendChild(node);
    return node;
}
